@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,7 +50,7 @@
 
 #define d(x) x
 
-int e_plugin_lib_enable (EPluginLib *ep, int enable);
+int e_plugin_lib_enable (EPlugin *ep, int enable);
 
 /* Account Setup */
 GtkWidget *org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData *data);
@@ -76,7 +77,7 @@ free_mapi_listener ( void )
 }
 
 int
-e_plugin_lib_enable (EPluginLib *ep, int enable)
+e_plugin_lib_enable (EPlugin *ep, int enable)
 {
 	g_debug ("Loading Exchange MAPI Plugin \n");
 
@@ -94,6 +95,90 @@ exchange_mapi_accounts_peek_config_listener ()
 	return config_listener; 
 }
 
+enum {
+  COL_MAPI_FULL_NAME = 0,
+  COL_MAPI_ACCOUNT,
+  COL_MAPI_INDEX,
+  COLS_MAX
+};
+
+/* Callback for ProcessNetworkProfile. If we have more than one username, 
+ we need to let the user select. */
+static uint32_t
+create_profile_callback (struct SRowSet *rowset, gpointer data)
+{
+	struct SPropValue *lpProp_fullname, *lpProp_account;
+	gint response;
+	guint32	i, index = 0;
+	GtkTreeIter iter;
+	GtkListStore *store;
+	GtkCellRenderer *renderer;
+	GtkTreeSelection *selection;
+	GtkWidget *dialog, *view;
+	GtkVBox *vbox;
+
+	/* NOTE: A good way would be display the list of username entries */
+	/* using GtkEntryCompletion in the username gtkentry. But plugins */
+	/* as of now does not have access to it */
+
+	/*TODO : Fix strings*/
+	dialog = gtk_dialog_new_with_buttons (_("Select username"),
+					      NULL, GTK_DIALOG_MODAL,
+					      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					      NULL);
+
+	/*Tree View */
+	view = gtk_tree_view_new ();
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
+						     -1, _("Full name"), renderer,
+						     "text", COL_MAPI_FULL_NAME, NULL);
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view),
+						     -1, _("User name"), renderer, 
+						     "text", COL_MAPI_ACCOUNT, NULL);
+
+	/* Model for TreeView */
+	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
+	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
+
+	for (i = 0; i < rowset->cRows; i++) {
+		lpProp_fullname = get_SPropValue_SRow(&(rowset->aRow[i]), PR_DISPLAY_NAME);
+		lpProp_account = get_SPropValue_SRow(&(rowset->aRow[i]), PR_ACCOUNT);
+
+		if (lpProp_fullname && lpProp_fullname->value.lpszA &&
+		    lpProp_account && lpProp_account->value.lpszA) {
+			gtk_list_store_append (store, &iter);
+			/* Preserve the index inside the store*/
+			gtk_list_store_set (store, &iter,
+					    COL_MAPI_FULL_NAME, lpProp_fullname->value.lpszA,
+					    COL_MAPI_ACCOUNT, lpProp_account->value.lpszA,
+					    COL_MAPI_INDEX, i, -1);
+		}
+	}
+
+	/* Pack the TreeView into dialog's content area */
+	vbox = (GtkVBox *)gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	gtk_box_pack_start (GTK_BOX (vbox), view, TRUE, TRUE, 6);
+	gtk_widget_show_all (GTK_WIDGET (vbox));
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (response == GTK_RESPONSE_ACCEPT) {
+	       /* Get the index from the selected value */
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+		gtk_tree_selection_get_selected (selection, NULL, &iter);
+		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX, 
+				    &index, -1);
+	} else /* If we return a value > available, we are canceling the login.*/
+	       index = rowset->cRows + 1;
+
+	gtk_widget_destroy (dialog);
+
+	return index;
+}
+
 static void
 validate_credentials (GtkWidget *widget, EConfig *config)
 {
@@ -105,6 +190,17 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 	url = camel_url_new (e_account_get_string (target_account->account, E_ACCOUNT_SOURCE_URL), NULL);
 	domain_name = camel_url_get_param (url, "domain");
 
+	/* Silently remove domain part from a user name when user enters it as such.
+	   This change will be visible in the UI on new edit open. */
+	if (url->user && strchr (url->user, '\\')) {
+		gchar *tmp, *at;
+
+		at = strrchr (url->user, '\\') + 1;
+		tmp = g_strdup (at);
+		camel_url_set_user (url, tmp);
+		g_free (tmp);
+	}
+
 	if (!url->user || !*url->user || !url->host || !*url->host || !domain_name || !*domain_name) {
 		e_notice (NULL, GTK_MESSAGE_ERROR, "%s", _("Server, username and domain name cannot be empty. Please fill them with correct values."));
 		return;
@@ -112,10 +208,11 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 
 	key = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD | CAMEL_URL_HIDE_PARAMS);
 	password = e_passwords_get_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
-	if (!password) {
+	if (!password || !*password) {
 		gboolean remember = FALSE;
 		gchar *title;
 
+		g_free (password);
 		title = g_strdup_printf (_("Enter Password for %s@%s"), url->user, url->host);
 		password = e_passwords_ask_password (title, EXCHANGE_MAPI_PASSWORD_COMPONENT, key, title,
 						     E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
@@ -126,12 +223,15 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 	/*Can there be a account without password ?*/
 	if (password && *password && domain_name && *domain_name && *url->user && *url->host) {
 		char *error_msg = NULL;
-		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name, url->host, &error_msg);
+		gboolean status = exchange_mapi_create_profile (url->user, password, domain_name,
+								url->host, &error_msg, 
+								(mapi_profile_callback_t) create_profile_callback,
+								NULL);
 		if (status) {
 			/* Things are successful */
 			gchar *profname = NULL, *uri = NULL; 
 
-			profname = g_strdup_printf("%s@%s", url->user, domain_name);
+			profname = exchange_mapi_util_profile_name (url->user, domain_name);
 			camel_url_set_param(url, "profile", profname);
 			g_free (profname);
 
@@ -154,6 +254,9 @@ validate_credentials (GtkWidget *widget, EConfig *config)
 		}
 
 		g_free (error_msg);
+	} else {
+		e_passwords_forget_password (EXCHANGE_MAPI_PASSWORD_COMPONENT, key);
+		e_notice (NULL, GTK_MESSAGE_ERROR, "%s", _("Authentication failed."));
 	}
 
 	g_free (password);
@@ -326,7 +429,6 @@ add_folders (GSList *folders, GtkTreeStore *ts)
 	gtk_tree_store_set (ts, &iter, 0, node, -1);
 	while (tmp) {
 		ExchangeMAPIFolder *folder = tmp->data;
-		g_print("%s\n", folder->folder_name);
 		add_to_store (ts, folder);
 		tmp = tmp->next;
 	}
@@ -361,13 +463,14 @@ exchange_mapi_create (EPlugin *epl, EConfigHookItemFactoryData *data)
 	GtkTreeStore *ts;
 	GtkTreeViewColumn *tvc;
 	const char *acc;
-	GSList *folders = exchange_mapi_account_listener_peek_folder_list ();
+	GSList *folders;
 
 	uri_text = e_source_get_uri (source);
 	if (uri_text && g_ascii_strncasecmp (uri_text, MAPI_URI_PREFIX, MAPI_PREFIX_LENGTH)) {
 		return NULL;
 	}
 
+	folders = exchange_mapi_account_listener_peek_folder_list ();
 	acc = e_source_group_peek_name (e_source_peek_group (source));
 	ts = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_INT64, G_TYPE_POINTER);
 
@@ -442,9 +545,7 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 {
 	EABConfigTargetSource *t = (EABConfigTargetSource *) target;
 	ESource *source = t->source;
-	char *uri_text, *tmp;
-	const char *sfid; 
-	mapi_id_t fid, pfid;
+	char *uri_text;
 	ESourceGroup *grp;
 	
 	uri_text = e_source_get_uri (source);
@@ -452,11 +553,6 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 		return;
 	
 	//FIXME: Offline handling
-	sfid = e_source_get_property (source, "parent-fid");
-	exchange_mapi_util_mapi_id_from_string (sfid, &pfid);
-
-	fid = exchange_mapi_create_folder (olFolderContacts, pfid, e_source_peek_name (source));
-	g_print("Created %016" G_GUINT64_FORMAT "X\n", fid);
 	grp = e_source_peek_group (source);
 	e_source_set_property (source, "auth", "plain/password");
 	e_source_set_property (source, "auth-domain", EXCHANGE_MAPI_PASSWORD_COMPONENT);
@@ -466,9 +562,6 @@ exchange_mapi_book_commit (EPlugin *epl, EConfigTarget *target)
 	e_source_set_property(source, "domain", e_source_group_get_property (grp, "domain"));
 	e_source_set_relative_uri (source, g_strconcat (";",e_source_peek_name (source), NULL));
 
-	tmp = exchange_mapi_util_mapi_id_to_string (fid);
-	e_source_set_property(source, "folder-id", tmp);
-	g_free (tmp);
 	e_source_set_property (source, "completion", "true");
 	// Update the folder list in the plugin and ExchangeMAPIFolder
 
@@ -593,4 +686,3 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	// Update the folder list in the plugin and ExchangeMAPIFolder
 	return;
 }
-
