@@ -538,7 +538,7 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
  				gchar *formatted_id = NULL;
 				const char *name, *display_name;
  				guint32 *type = NULL;
- 				struct SRow aRow;
+ 				struct SRow *aRow;
  				ExchangeMAPIRecipient *recip = (ExchangeMAPIRecipient *)(l->data);
  				
  				/*Can't continue when there is no email-id*/
@@ -546,18 +546,16 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
  					continue;
  				
  				/* Build a SRow structure */
- 				aRow.ulAdrEntryPad = 0;
- 				aRow.cValues = recip->out.all_cValues;
- 				aRow.lpProps = recip->out.all_lpProps;
+ 				aRow = &recip->out_SRow;
 
- 				type = (uint32_t *) find_SPropValue_data(&aRow, PR_RECIPIENT_TYPE);
+ 				type = (uint32_t *) find_SPropValue_data (aRow, PR_RECIPIENT_TYPE);
 
 				if (type) {
- 					name = (const char *) find_SPropValue_data(&aRow, PR_DISPLAY_NAME);
- 					name = name ? name : (const char *) find_SPropValue_data(&aRow, PR_RECIPIENT_DISPLAY_NAME);
- 					name = name ? name : (const char *) find_SPropValue_data(&aRow, 
+ 					name = (const char *) find_SPropValue_data (aRow, PR_DISPLAY_NAME);
+ 					name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME);
+ 					name = name ? name : (const char *) find_SPropValue_data (aRow, 
  												 PR_RECIPIENT_DISPLAY_NAME_UNICODE);
- 					name = name ? name : (const char *) find_SPropValue_data(&aRow, 
+ 					name = name ? name : (const char *) find_SPropValue_data (aRow, 
  												 PR_7BIT_DISPLAY_NAME_UNICODE);
  					display_name = name ? name : recip->email_id;
  					formatted_id = camel_internet_address_format_address(display_name, recip->email_id);
@@ -656,6 +654,7 @@ struct mapi_update_deleted_msg {
 
 	CamelFolder *folder;
 	mapi_id_t folder_id;
+	gboolean need_refresh;
 };
 
 static gboolean
@@ -729,8 +728,8 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	for (index = 0; index < count; index++) {
 		GSList *tmp_list_item = NULL;
 
-		/*FIXME :Any other list available ???*/
-		info = camel_folder_summary_index (m->folder->summary, index);
+		/* Iterate in a reverse order, thus removal will not hurt */
+		info = camel_folder_summary_index (m->folder->summary, count - index - 1);
 		if (!info) continue; /*This is bad. *Should* not happen*/
 
 		uid = camel_message_info_uid (info);
@@ -763,6 +762,8 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	camel_object_trigger_event (m->folder, "folder_changed", changes);
 	camel_folder_change_info_free (changes);
 
+	m->need_refresh = camel_folder_summary_count (m->folder->summary) != g_slist_length (server_uid_list);
+
 	/* Discard server uid list */
 	g_slist_foreach (server_uid_list, (GFunc) g_free, NULL);
 	g_slist_free (server_uid_list);
@@ -773,8 +774,25 @@ mapi_sync_deleted_free (CamelSession *session, CamelSessionThreadMsg *msg)
 {
 	struct mapi_update_deleted_msg *m = (struct mapi_update_deleted_msg *)msg;
 
+	if (m->need_refresh) {
+		CamelMapiSummary *mapi_summary = CAMEL_MAPI_SUMMARY (m->folder->summary);
+		if (mapi_summary) {
+			CamelException ex = CAMEL_EXCEPTION_INITIALISER;
+
+			CAMEL_SERVICE_REC_LOCK (m->folder->parent_store, connect_lock);
+			g_free (mapi_summary->sync_time_stamp);
+			mapi_summary->sync_time_stamp = NULL;
+			CAMEL_SERVICE_REC_UNLOCK (m->folder->parent_store, connect_lock);
+
+			mapi_refresh_folder (m->folder, &ex);
+
+			if (camel_exception_is_set (&ex))
+				g_warning ("%s: %s", G_STRFUNC, ex.desc);
+			camel_exception_clear (&ex);
+		}
+	}
+
 	camel_object_unref (m->folder);
-	/* camel_session_thread_msg_free (session, &m->msg); */
 }
 
 static CamelSessionThreadOps deleted_items_sync_ops = {
@@ -952,15 +970,21 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		PR_MESSAGE_DELIVERY_TIME,
 		PR_MESSAGE_FLAGS,
 		PR_SENT_REPRESENTING_NAME,
+		PR_SENT_REPRESENTING_NAME_UNICODE,
 		PR_SENT_REPRESENTING_EMAIL_ADDRESS,
+		PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE,
 		PR_SENT_REPRESENTING_ADDRTYPE,
+		PR_SENT_REPRESENTING_ADDRTYPE_UNICODE,
 		PR_LAST_MODIFICATION_TIME,
 		PR_INTERNET_MESSAGE_ID,
 		PR_INTERNET_REFERENCES,
 		PR_IN_REPLY_TO_ID,
 		PR_DISPLAY_TO,
+		PR_DISPLAY_TO_UNICODE,
 		PR_DISPLAY_CC,
+		PR_DISPLAY_CC_UNICODE,
 		PR_DISPLAY_BCC,
+		PR_DISPLAY_BCC_UNICODE,
 		PR_TRANSPORT_MESSAGE_HEADERS,
 		PR_TRANSPORT_MESSAGE_HEADERS_UNICODE
 	};
@@ -1068,6 +1092,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 							 sizeof (*deleted_items_op_msg));
 		deleted_items_op_msg->folder = folder;
 		deleted_items_op_msg->folder_id = temp_folder_id;
+		deleted_items_op_msg->need_refresh = FALSE;
 		camel_object_ref (folder);
 
 		camel_session_thread_queue (session, &deleted_items_op_msg->msg, 0);
@@ -1293,10 +1318,13 @@ fetch_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 		item->msg.body_parts = g_slist_append (item->msg.body_parts, body);
 
 		item->is_cal = TRUE;
+
+		g_free (appointment_body_str);
 	} else { 
-		if (!((body = exchange_mapi_util_find_stream (item_data->streams, PR_HTML)) || 
+		/* always prefer unicode version, as that can be properly read */
+		if (!((body = exchange_mapi_util_find_stream (item_data->streams, PR_BODY_UNICODE)) || 
 		      (body = exchange_mapi_util_find_stream (item_data->streams, PR_BODY))))
-			body = exchange_mapi_util_find_stream (item_data->streams, PR_BODY_UNICODE);
+			body = exchange_mapi_util_find_stream (item_data->streams, PR_HTML);
 
 		item->msg.body_parts = g_slist_append (item->msg.body_parts, body);
 
@@ -1340,7 +1368,7 @@ mapi_msg_set_recipient_list (CamelMimeMessage *msg, MapiItem *item)
 		const char *name = NULL;
 		uint32_t rcpt_type = MAPI_TO;
 		uint32_t *type = NULL; 
-		struct SRow aRow;
+		struct SRow *aRow;
 		ExchangeMAPIRecipient *recip = (ExchangeMAPIRecipient *)(l->data);
 		
 		/*Can't continue when there is no email-id*/
@@ -1348,17 +1376,15 @@ mapi_msg_set_recipient_list (CamelMimeMessage *msg, MapiItem *item)
 			continue;
 		
 		/* Build a SRow structure */
-		aRow.ulAdrEntryPad = 0;
-		aRow.cValues = recip->out.all_cValues;
-		aRow.lpProps = recip->out.all_lpProps;
+		aRow = &recip->out_SRow;
 		
 		/*Name is probably available in one of these props.*/
-		name = (const char *) find_SPropValue_data(&aRow, PR_DISPLAY_NAME);
-		name = name ? name : (const char *) find_SPropValue_data(&aRow, PR_RECIPIENT_DISPLAY_NAME);
-		name = name ? name : (const char *) find_SPropValue_data(&aRow, PR_RECIPIENT_DISPLAY_NAME_UNICODE);
-		name = name ? name : (const char *) find_SPropValue_data(&aRow, PR_7BIT_DISPLAY_NAME_UNICODE);
+		name = (const char *) find_SPropValue_data (aRow, PR_DISPLAY_NAME);
+		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME);
+		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME_UNICODE);
+		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_7BIT_DISPLAY_NAME_UNICODE);
 
-		type = (uint32_t *) find_SPropValue_data(&aRow, PR_RECIPIENT_TYPE);
+		type = (uint32_t *) find_SPropValue_data (aRow, PR_RECIPIENT_TYPE);
 		
 		/*Fallbacks. Not good*/
 		display_name = name ? g_strdup (name) : g_strdup (recip->email_id);
@@ -1416,6 +1442,11 @@ mapi_populate_details_from_item (CamelFolder *folder, CamelMimeMessage *msg, Map
 			for (h = part->headers; h; h = h->next) {
 				const gchar *value = h->value;
 
+				/* skip all headers describing content of a message,
+				   because it's overwritten on message decomposition */
+				if (g_ascii_strncasecmp (h->name, "Content", 7) == 0)
+					continue;
+
 				while (value && camel_mime_is_lwsp (*value))
 					value++;
 
@@ -1463,15 +1494,12 @@ mapi_populate_details_from_item (CamelFolder *folder, CamelMimeMessage *msg, Map
 
 
 static void
-mapi_populate_msg_body_from_item (CamelMultipart *multipart, MapiItem *item, ExchangeMAPIStream *body)
+mapi_populate_msg_body_from_item (CamelMimePart *part, MapiItem *item, ExchangeMAPIStream *body)
 {
-	CamelMimePart *part;
-	const char* type = NULL;
-
-	part = camel_mime_part_new ();
-	camel_mime_part_set_encoding(part, CAMEL_TRANSFER_ENCODING_8BIT);
+	camel_mime_part_set_encoding (part, CAMEL_TRANSFER_ENCODING_8BIT);
 	
 	if (body) {
+		const gchar* type = NULL;
 		gchar *buff = NULL;
 
 		if (item->is_cal)
@@ -1491,14 +1519,36 @@ mapi_populate_msg_body_from_item (CamelMultipart *multipart, MapiItem *item, Exc
 		camel_mime_part_set_content (part, (const char *) body->value->data, body->value->len, type);
 
 		g_free (buff);
-		type = NULL;
 	} else
-		camel_mime_part_set_content(part, " ", strlen(" "), "text/html");
-
-	camel_multipart_add_part (multipart, part);
-	camel_object_unref (part);
+		camel_mime_part_set_content (part, "", strlen (""), "text/plain");
 }
 
+static gint
+sort_bodies_cb (gconstpointer a, gconstpointer b)
+{
+	static const gint desired_order[] = { PR_BODY, PR_BODY_UNICODE, PR_HTML };
+	const ExchangeMAPIStream *stream_a = a, *stream_b = b;
+	gint aidx, bidx;
+
+	if (a == b)
+		return 0;
+	if (!a)
+		return -1;
+	if (!b)
+		return 1;
+
+	for (aidx = 0; aidx < G_N_ELEMENTS (desired_order); aidx++) {
+		if (desired_order[aidx] == stream_a->proptag)
+			break;
+	}
+
+	for (bidx = 0; bidx < G_N_ELEMENTS (desired_order); bidx++) {
+		if (desired_order[bidx] == stream_b->proptag)
+			break;
+	}
+
+	return aidx - bidx;
+}
 
 static CamelMimeMessage *
 mapi_folder_item_to_msg( CamelFolder *folder,
@@ -1507,32 +1557,14 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 {
 	CamelMimeMessage *msg = NULL;
 	CamelMultipart *multipart = NULL;
+	CamelMimePart *part = NULL;
+	CamelDataWrapper *msg_content = NULL;
 
 	GSList *attach_list = NULL;
-	/* int errno; */
-	/* char *body = NULL; */
-	ExchangeMAPIStream *body = NULL;
 	GSList *body_part_list = NULL;
 
 	attach_list = item->attachments;
 	msg = camel_mime_message_new ();
-	multipart = camel_multipart_new ();
-
-	/*FIXME : Using set of default. Fix it during mimewriter*/
-	camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (multipart),
-					  "multipart/related");
-	camel_content_type_set_param(CAMEL_DATA_WRAPPER (multipart)->mime_type, 
-				     "type", "multipart/alternative");
-
-	camel_multipart_set_boundary (multipart, NULL);
-
-	/* Handle Multipart */
-	body_part_list = item->msg.body_parts;
-	while (body_part_list){
-	       body = body_part_list->data;
-	       mapi_populate_msg_body_from_item (multipart, item, body);	       
-	       body_part_list = g_slist_next (body_part_list);
-	}
 
 	/* Threading */
 	if (item->header.message_id)
@@ -1544,18 +1576,69 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 	if (item->header.in_reply_to)
 		camel_medium_add_header (CAMEL_MEDIUM (msg), "In-Reply-To", item->header.in_reply_to);
 
-	/*Set recipient details*/
+	/* Set recipient details */
 	mapi_msg_set_recipient_list (msg, item);
 	mapi_populate_details_from_item (folder, msg, item);
 
-	/** Attachment Handling*/
+	if (g_slist_length (attach_list) > 0) {
+		multipart = camel_multipart_new ();
+		camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (multipart), "multipart/mixed");
+		camel_multipart_set_boundary (multipart, NULL);
+
+		msg_content = CAMEL_DATA_WRAPPER (multipart);
+	}
+
+	if (g_slist_length (item->msg.body_parts) > 1) {
+		multipart = camel_multipart_new ();
+		camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (multipart), "multipart/alternative");
+		camel_multipart_set_boundary (multipart, NULL);
+
+		item->msg.body_parts = g_slist_sort (item->msg.body_parts, sort_bodies_cb);
+
+		/* Handle Multipart */
+		for (body_part_list = item->msg.body_parts; body_part_list; body_part_list = g_slist_next (body_part_list)) {
+			part = camel_mime_part_new ();
+
+			mapi_populate_msg_body_from_item (part, item, body_part_list->data);
+
+			camel_multipart_add_part (CAMEL_MULTIPART (msg_content), part);
+			camel_object_unref (part);
+		}
+
+		if (msg_content) {
+			part = camel_mime_part_new ();
+			camel_data_wrapper_set_mime_type (CAMEL_DATA_WRAPPER (part), "multipart/alternative");
+			camel_medium_set_content_object (CAMEL_MEDIUM (part), CAMEL_DATA_WRAPPER (multipart));
+			camel_object_unref (multipart);
+
+			camel_multipart_add_part (CAMEL_MULTIPART (msg_content), part);
+			camel_object_unref (part);
+		} else {
+			msg_content = CAMEL_DATA_WRAPPER (multipart);
+		}
+	} else {
+		if (msg_content) {
+			part = camel_mime_part_new ();
+			mapi_populate_msg_body_from_item (part, item, item->msg.body_parts ? item->msg.body_parts->data : NULL);
+			camel_multipart_add_part (CAMEL_MULTIPART (msg_content), part);
+			camel_object_unref (part);
+		} else {
+			msg_content = CAMEL_DATA_WRAPPER (msg);
+			mapi_populate_msg_body_from_item (CAMEL_MIME_PART (msg), item, item->msg.body_parts ? item->msg.body_parts->data : NULL);
+		}
+	}
+
+	/* Attachment Handling */
 	if (attach_list) {
-		GSList *al = attach_list;
+		GSList *al;
+
+		multipart = CAMEL_MULTIPART (msg_content);
+
 		for (al = attach_list; al != NULL; al = al->next) {
 			ExchangeMAPIAttachment *attach = (ExchangeMAPIAttachment *)al->data;
 			ExchangeMAPIStream *stream = NULL;
 			const char *filename, *mime_type, *content_id = NULL; 
-			CamelMimePart *part;
+			CamelContentType *content_type;
 
 			stream = exchange_mapi_util_find_stream (attach->streams, PR_ATTACH_DATA_BIN);
 
@@ -1575,8 +1658,9 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 			camel_content_type_set_param (((CamelDataWrapper *) part)->mime_type, "name", filename);
 
 			/*Content-Type*/
-			mime_type = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
-												  PR_ATTACH_MIME_TAG);
+			mime_type = (const char *) exchange_mapi_util_find_SPropVal_array_propval (attach->lpProps, PR_ATTACH_MIME_TAG);
+			if (!mime_type)
+				mime_type = "application/octet-stream";
 
 			camel_mime_part_set_content (part, (const char *) stream->value->data, stream->value->len, mime_type);
 
@@ -1586,8 +1670,11 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 
 			camel_mime_part_set_content_id (part, content_id);
 
-			/*FIXME : Mime Reader / Writer work*/
-			//camel_mime_part_set_encoding (part, CAMEL_TRANSFER_ENCODING_BASE64);
+			content_type = camel_mime_part_get_content_type (part);
+			if (content_type && camel_content_type_is (content_type, "text", "*"))
+				camel_mime_part_set_encoding (part, CAMEL_TRANSFER_ENCODING_QUOTEDPRINTABLE);
+			else
+				camel_mime_part_set_encoding (part, CAMEL_TRANSFER_ENCODING_BASE64);
 
 			camel_multipart_add_part (multipart, part);
 			camel_object_unref (part);
@@ -1596,16 +1683,14 @@ mapi_folder_item_to_msg( CamelFolder *folder,
 		exchange_mapi_util_free_attachment_list (&attach_list);
 	}
 
-	camel_medium_set_content_object(CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER(multipart));
-	camel_object_unref (multipart);
-
-	if (body)
-		g_free (body);
+	if (msg_content != CAMEL_DATA_WRAPPER (msg)) {
+		camel_medium_set_content_object (CAMEL_MEDIUM (msg), msg_content);
+		camel_object_unref (msg_content);
+	}
 
 	return msg;
 }
 
-
 static CamelMimeMessage *
 mapi_folder_get_message( CamelFolder *folder, const char *uid, CamelException *ex )
 {
@@ -1909,12 +1994,12 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, source->full_name) ;
 	exchange_mapi_util_mapi_id_from_string (folder_id, &src_fid);
 
-	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, destination->full_name) ;
+	folder_id = camel_mapi_store_folder_id_lookup (mapi_store, destination->full_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &dest_fid);
 
 	for (i=0; i < uids->len; i++) {
 		mapi_id_t *mid = g_new0 (mapi_id_t, 1); /* FIXME : */
-		if (!exchange_mapi_util_mapi_ids_from_uid (g_ptr_array_index (uids, i), &src_fid, mid)) 
+		if (!exchange_mapi_util_mapi_ids_from_uid (g_ptr_array_index (uids, i), &src_fid, mid))
 			continue;
 
 		src_msg_ids = g_slist_prepend (src_msg_ids, mid);
@@ -2009,7 +2094,8 @@ mapi_append_message (CamelFolder *folder, CamelMimeMessage *message,
 					 item->recipients, item->attachments,
 					 item->generic_streams, 0);
 
-	*appended_uid = exchange_mapi_util_mapi_ids_to_uid(fid, mid);
+	if (appended_uid)
+		*appended_uid = exchange_mapi_util_mapi_ids_to_uid(fid, mid);
 }
 
 static void
