@@ -64,33 +64,67 @@ static GStaticRecMutex connect_lock = G_STATIC_REC_MUTEX_INIT;
 #define STREAM_ACCESS_WRITE     0x0001
 #define STREAM_ACCESS_READWRITE 0x0002
 
+static gboolean
+ensure_mapi_init_called (void)
+{
+	static gboolean called = FALSE;
+	gchar *profpath;
+	enum MAPISTATUS status;
+
+	LOCK ();
+	if (called) {
+		UNLOCK ();
+		return TRUE;
+	}
+
+	profpath = g_build_filename (g_get_home_dir (), DEFAULT_PROF_PATH, NULL);
+
+	if (!g_file_test (profpath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
+		/* Create a ProfileStore */
+		status = CreateProfileStore (profpath, LIBMAPI_LDIF_DIR);
+		if (status != MAPI_E_SUCCESS && (status != MAPI_E_NO_ACCESS || !g_file_test (profpath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))) {
+			mapi_errstr ("CreateProfileStore", GetLastError());
+			g_free (profpath);
+
+			UNLOCK ();
+			return FALSE;
+		}
+	}
+
+	status = MAPIInitialize (profpath);
+	if (status == MAPI_E_SESSION_LIMIT) {
+		/* do nothing, the profile store is already initialized */
+		/* but this shouldn't happen */
+		mapi_errstr ("MAPIInitialize", GetLastError());
+	} else if (status != MAPI_E_SUCCESS) {
+		mapi_errstr ("MAPIInitialize", GetLastError());
+		g_free (profpath);
+
+		UNLOCK ();
+		return FALSE;
+	}
+
+	g_free (profpath);
+
+	called = TRUE;
+	UNLOCK ();
+
+	return TRUE;
+}
+
 static struct mapi_session *
 mapi_profile_load (const char *profname, const char *password)
 {
 	enum MAPISTATUS	retval = MAPI_E_SUCCESS;
 	struct mapi_session *session = NULL;
-	gchar *profpath = NULL;
 	gchar *default_profile_name = NULL;
 	const char *profile = NULL;
 	guint32 debug_log_level = 0;
 
 	d(g_print("\n%s: Entering %s ", G_STRLOC, G_STRFUNC));
 
-	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
-	if (!g_file_test (profpath, G_FILE_TEST_EXISTS)) {
-		g_warning ("\nMAPI profile database @ %s not found ", profpath);
+	if (!ensure_mapi_init_called ())
 		goto cleanup;
-	}
-
-	MAPIUninitialize ();
-
-	retval = MAPIInitialize(profpath);
-	if (retval != MAPI_E_SUCCESS) {
-		mapi_errstr("MAPIInitialize", GetLastError());
-		if (retval == MAPI_E_SESSION_LIMIT)
-			g_print("\n%s: %s: Already connected ", G_STRLOC, G_STRFUNC);
-		goto cleanup;
-	}
 
 	if (g_getenv ("MAPI_DEBUG")) {
 		debug_log_level = atoi (g_getenv ("MAPI_DEBUG"));
@@ -117,12 +151,6 @@ mapi_profile_load (const char *profname, const char *password)
 	}
 
 cleanup:
-	if (retval != MAPI_E_SUCCESS && retval != MAPI_E_SESSION_LIMIT &&
-	    retval != MAPI_E_LOGON_FAILED)
-		MAPIUninitialize ();
-
-	g_free (profpath);
-
 	d(g_print("\n%s: Leaving %s ", G_STRLOC, G_STRFUNC));
 
 	return session;
@@ -151,13 +179,27 @@ exchange_mapi_connection_new (const char *profile, const char *password)
 }
 
 void
-exchange_mapi_connection_close ()
+exchange_mapi_connection_close (void)
 {
 	LOCK();
+
+	if (global_mapi_session) {
+		mapi_object_t obj_store;
+		enum MAPISTATUS status;
+
+		mapi_object_init (&obj_store);
+
+		/* Open the message store */
+		status = OpenMsgStore (global_mapi_session, &obj_store);
+		if (status != MAPI_E_SUCCESS) {
+			mapi_errstr ("OpenMsgStore", GetLastError());
+		} else {
+			Logoff (&obj_store);
+		}
+	}
 	global_mapi_session = NULL;
-	MAPIUninitialize ();	
+
 	UNLOCK();
-	/* TODO :  Return status. get last error ? */
 }
 
 static gboolean 
@@ -231,7 +273,7 @@ exchange_mapi_util_read_generic_stream (mapi_object_t *obj_message, uint32_t pro
 
 		/* Build a mapi_SPropValue_array structure */
 		properties_array.cValues = 1; 
-		properties_array.lpProps = talloc_array (mem_ctx, struct mapi_SPropValue, properties_array.cValues);
+		properties_array.lpProps = talloc_zero_array (mem_ctx, struct mapi_SPropValue, properties_array.cValues + 1);
 		properties_array.lpProps[0].ulPropTag = proptag; 
 		/* This call is needed in case the read stream was a named prop. */
 		mapi_SPropValue_array_named (obj_message, &properties_array);
@@ -750,6 +792,8 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 	
 	mem_ctx = talloc_init ("ExchangeMAPI_GetGAL");
 
+	LOCK ();
+
 	SPropTagArray = set_SPropTagArray(mem_ctx, 0xc,
 					  PR_INSTANCE_KEY,
 					  PR_ENTRYID,
@@ -768,9 +812,12 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 	ulFlags = TABLE_START;
 	do {
 		count += 0x2;
+		SRowSet = NULL;
 		retval = GetGALTable(global_mapi_session, SPropTagArray, &SRowSet, count, ulFlags);
-		if ((!SRowSet) || (!(SRowSet->aRow))) {
-			return false;
+		if ((!SRowSet) || (!(SRowSet->aRow)) || retval != MAPI_E_SUCCESS) {
+			UNLOCK ();
+			MAPIFreeBuffer (SPropTagArray);
+			return FALSE;
 		}
 		if (SRowSet->cRows) {
 			for (i = 0; i < SRowSet->cRows; i++) {
@@ -786,7 +833,9 @@ exchange_mapi_util_get_gal (GPtrArray *contacts_array)
 
 	MAPIFreeBuffer(SPropTagArray);
 
-	return true;
+	UNLOCK ();
+
+	return TRUE;
 	
 }
 
@@ -829,8 +878,9 @@ exchange_mapi_util_get_recipients (mapi_object_t *obj_message, GSList **recip_li
 			mapidump_SRow (&(rows_recip.aRow[i_row_recip]), " ");
 		}
 
-		recipient->out.all_lpProps = talloc_steal ((TALLOC_CTX *)recipient->mem_ctx, rows_recip.aRow[i_row_recip].lpProps);
-		recipient->out.all_cValues = rows_recip.aRow[i_row_recip].cValues;
+		recipient->out_SRow.ulAdrEntryPad = rows_recip.aRow[i_row_recip].ulAdrEntryPad;
+		recipient->out_SRow.cValues = rows_recip.aRow[i_row_recip].cValues;
+		recipient->out_SRow.lpProps = talloc_steal ((TALLOC_CTX *)recipient->mem_ctx, rows_recip.aRow[i_row_recip].lpProps);
 
 		*recip_list = g_slist_append (*recip_list, recipient);
 	}
@@ -908,7 +958,9 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 	}
 
 	/* Attempt to resolve names from the server */
+	LOCK ();
 	retval = ResolveNames (global_mapi_session, users, SPropTagArray, &SRowSet, &FlagList, 0);
+	UNLOCK ();
 	if (retval != MAPI_E_SUCCESS) {
 		mapi_errstr("ResolveNames", GetLastError());
 		goto cleanup;
@@ -936,7 +988,10 @@ exchange_mapi_util_modify_recipients (TALLOC_CTX *mem_ctx, mapi_object_t *obj_me
 			set_recipient_properties(mem_ctx, &SRowSet->aRow[last], recipient, TRUE);
 			SRowSet->cRows += 1;
 		} else if (FlagList->aulPropTag[i] == MAPI_RESOLVED) {
-			set_recipient_properties (mem_ctx, &SRowSet->aRow[j], recipient, FALSE);
+			/* FIXME: Even though some recipients are resolved, their mailboxes need not reside 
+			on the same server in case of cluster setups. So lets set ext_lpProps for all recipients.
+			Lets have this fix until we have openchange handling this case better. */
+			set_recipient_properties (mem_ctx, &SRowSet->aRow[j], recipient, TRUE);
 			j += 1;
 		}
 	}
@@ -1115,7 +1170,7 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 	}
 
 	/* Get a handle on the container */
-	retval = GetContentsTable(&obj_folder, &obj_table, 0, NULL);
+	retval = GetContentsTable(&obj_folder, &obj_table, TableFlags_UseUnicode, NULL);
 	if (retval != MAPI_E_SUCCESS) {
 		mapi_errstr("GetContentsTable", GetLastError());
 		goto cleanup;
@@ -1211,7 +1266,7 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 		for (i = 0; i < SRowSet.cRows; i++) {
 			mapi_object_t obj_message;
-			struct mapi_SPropValue_array properties_array;
+			struct mapi_SPropValue_array properties_array = {0};
 			const mapi_id_t *pfid;
 			const mapi_id_t	*pmid;
 			const bool *has_attach = NULL;
@@ -1227,6 +1282,9 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 			has_attach = (const bool *) get_SPropValue_SRow_data(&SRowSet.aRow[i], PR_HASATTACH);
 
+			if (options & MAPI_OPTIONS_DONT_OPEN_MESSAGE)
+				goto relax;
+				
 			retval = OpenMessage(&obj_folder, *pfid, *pmid, &obj_message, 0);
 			if (retval != MAPI_E_SUCCESS) {
 				mapi_errstr("OpenMessage", GetLastError());
@@ -1255,26 +1313,28 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 
 				/* Conversion from SPropValue to mapi_SPropValue. (no padding here) */
 				properties_array.cValues = prop_count;
-				properties_array.lpProps = talloc_array (mem_ctx, struct mapi_SPropValue, 
-									 prop_count);
+				properties_array.lpProps = talloc_zero_array (mem_ctx, struct mapi_SPropValue, 
+									 prop_count + 1);
 				for (k=0; k < prop_count; k++)
 					cast_mapi_SPropValue(&properties_array.lpProps[k], &lpProps[k]);
 
 			} else
 				retval = GetPropsAll (&obj_message, &properties_array);
-
+ relax:
 			if (retval == MAPI_E_SUCCESS) {
 				FetchItemsCallbackData *item_data;
 				uint32_t z;
 
-				/* just to get all the other streams */
-				for (z=0; z < properties_array.cValues; z++) {
-					if ((properties_array.lpProps[z].ulPropTag & 0xFFFF) == PT_BINARY && 
-					    (options & MAPI_OPTIONS_FETCH_GENERIC_STREAMS)) 
-					exchange_mapi_util_read_generic_stream (&obj_message, properties_array.lpProps[z].ulPropTag, &stream_list);
-				}
+				if ((options & MAPI_OPTIONS_DONT_OPEN_MESSAGE) == 0) {
+					/* just to get all the other streams */
+					for (z=0; z < properties_array.cValues; z++) {
+						if ((properties_array.lpProps[z].ulPropTag & 0xFFFF) == PT_BINARY && 
+						    (options & MAPI_OPTIONS_FETCH_GENERIC_STREAMS)) 
+						exchange_mapi_util_read_generic_stream (&obj_message, properties_array.lpProps[z].ulPropTag, &stream_list);
+					}
 
-				mapi_SPropValue_array_named(&obj_message, &properties_array);
+					mapi_SPropValue_array_named(&obj_message, &properties_array);
+				}
 
 				/* NOTE: stream_list, recipient_list and attach_list
 				   should be freed by the callback */
@@ -1297,7 +1357,8 @@ exchange_mapi_connection_fetch_items   (mapi_id_t fid,
 				talloc_free (properties_array.lpProps);
 
 		loop_cleanup:
-			mapi_object_release(&obj_message);
+			if ((options & MAPI_OPTIONS_DONT_OPEN_MESSAGE) == 0)
+				mapi_object_release (&obj_message);
 
 			if (!cb_retval) break;
 		}
@@ -1432,7 +1493,7 @@ exchange_mapi_connection_fetch_item (mapi_id_t fid, mapi_id_t mid,
 
 		/* Conversion from SPropValue to mapi_SPropValue. (no padding here) */
 		properties_array.cValues = prop_count;
-		properties_array.lpProps = talloc_array (mem_ctx, struct mapi_SPropValue, prop_count);
+		properties_array.lpProps = talloc_zero_array (mem_ctx, struct mapi_SPropValue, prop_count + 1);
 		for (k=0; k < prop_count; k++)
 			cast_mapi_SPropValue(&properties_array.lpProps[k], &lpProps[k]);
 
@@ -2133,6 +2194,22 @@ exchange_mapi_create_item (uint32_t olFolder, mapi_id_t fid,
 		retval = SubmitMessage(&obj_message);
 		if (retval != MAPI_E_SUCCESS) {
 			mapi_errstr("SubmitMessage", GetLastError());
+
+			if ((options & MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE) != 0) {
+				mid = mapi_object_get_id (&obj_message);
+				mapi_object_release(&obj_message);
+				/* to not release a message object twice */
+				mapi_object_init (&obj_message);
+
+				retval = DeleteMessage (&obj_folder, &mid, 1);
+				if (retval != MAPI_E_SUCCESS) {
+					mapi_errstr ("DeleteMessage", GetLastError ());
+				}
+
+				/* do not forget to set it back to 0, as the function failed */
+				mid = 0;
+			}
+
 			goto cleanup;
 		}
 	}
@@ -2252,6 +2329,8 @@ exchange_mapi_modify_item (uint32_t olFolder, mapi_id_t fid, mapi_id_t mid,
 	/* Set attachments if any */
 	if (attachments) {
 		exchange_mapi_util_set_attachments (&obj_message, attachments, TRUE);
+	} else {
+		exchange_mapi_util_delete_attachments (&obj_message);
 	}
 
 	/* Set recipients if any */
@@ -2983,7 +3062,7 @@ exchange_mapi_create_profile (const char *username, const char *password, const
 	enum MAPISTATUS	retval;
 	gboolean result = FALSE; 
 	const gchar *workstation = "localhost";
-	gchar *profname = NULL, *profpath = NULL;
+	gchar *profname = NULL;
 	struct mapi_session *session = NULL;
 
 	/*We need all the params before proceeding.*/
@@ -2994,31 +3073,9 @@ exchange_mapi_create_profile (const char *username, const char *password, const
 
 	LOCK ();
 
-	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
-	profname = g_strdup_printf("%s@%s", username, domain);
+	profname = exchange_mapi_util_profile_name (username, domain);
 
-	if (!g_file_test (profpath, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
-		/* Create a ProfileStore */
-		retval = CreateProfileStore (profpath, LIBMAPI_LDIF_DIR); 
-		if (retval != MAPI_E_SUCCESS) {
-			manage_mapi_error ("CreateProfileStore", GetLastError(), error_msg);
-			g_free (profpath);
-			g_free (profname);
-			
-			UNLOCK ();
-			return FALSE;
-		}
-	}
-
-	retval = MAPIInitialize(profpath); 
-	if (retval == MAPI_E_SESSION_LIMIT)
-	/* do nothing, the profile store is already initialized */
-		manage_mapi_error ("MAPIInitialize", GetLastError(), error_msg); 
-	else if (retval != MAPI_E_SUCCESS) {
-		manage_mapi_error ("MAPIInitialize", GetLastError(), error_msg);
-		g_free (profpath);
-		g_free (profname);
-		
+	if (!ensure_mapi_init_called ()) {
 		UNLOCK ();
 		return FALSE;
 	}
@@ -3081,13 +3138,9 @@ exchange_mapi_create_profile (const char *username, const char *password, const
 	} else 
 		goto exit;
 
-cleanup: 
-	if (!result)
-		MAPIUninitialize ();
-
-exit:
+ cleanup: 
+ exit:
 	g_free (profname);
-	g_free (profpath);
 
 	UNLOCK ();
 
@@ -3099,23 +3152,10 @@ exchange_mapi_delete_profile (const char *profile)
 {
 	enum MAPISTATUS	retval;
 	gboolean result = FALSE; 
-	gchar *profpath = NULL;
 
 	LOCK ();
 
-	profpath = g_build_filename (g_get_home_dir(), DEFAULT_PROF_PATH, NULL);
-	if (!g_file_test (profpath, G_FILE_TEST_EXISTS)) {
-		g_warning ("No need to delete profile. DB itself is missing \n");
-		result = TRUE;
-		goto cleanup; 
-	}
-
-	retval = MAPIInitialize(profpath); 
-	if (retval == MAPI_E_SESSION_LIMIT)
-	/* do nothing, the profile store is already initialized */
-		; 
-	else if (retval != MAPI_E_SUCCESS) {
-		mapi_errstr("MAPIInitialize", GetLastError());
+	if (!ensure_mapi_init_called ()) {
 		goto cleanup; 
 	}
 
@@ -3130,8 +3170,6 @@ exchange_mapi_delete_profile (const char *profile)
 	result = TRUE; 
 
 cleanup: 
-	g_free(profpath);
-
 	UNLOCK ();
 
 	return result;
