@@ -46,9 +46,26 @@ utf8tolinux (const char *wstring)
 {
 	TALLOC_CTX 	*mem_ctx;
 	gchar		*newstr, *retval = NULL;
+	gint i;
+	gboolean all_ok = TRUE;
 
 	g_return_val_if_fail (wstring != NULL, NULL);
 
+	/* If all letters are with code <128 then there is nothing
+	   to be converted, thus return the original string copy.
+	   It's not working always, but should be fine for now.
+
+	   This is a temporary hack before the windows_to_utf8 will
+	   be fixed to not drop some letters.
+	*/
+	for (i = 0; all_ok && wstring[i]; i++) {
+		/* signed char, thus '> 128' means '< 0' */
+		all_ok = wstring[i] > 0;
+	}
+
+	if (all_ok)
+		return g_strdup (wstring);
+
 	mem_ctx = talloc_init ("ExchangeMAPI_utf8tolinux");
 
 	newstr = windows_to_utf8(mem_ctx, wstring);
@@ -270,8 +287,8 @@ exchange_mapi_util_free_recipient_list (GSList **recip_list)
 			g_free (recipient->in.ext_lpProps);
 		if (recipient->in.req_cValues)
 			g_free (recipient->in.req_lpProps);
-/*		if (recipient->out.all_cValues)
-			g_free (recipient->out.all_lpProps);
+/*		if (recipient->out_SRow.cValues)
+			g_free (recipient->out_SRow.lpProps);
 */		g_free (recipient);
 	}
 	g_slist_free (l);
@@ -576,3 +593,20 @@ exchange_crlf_to_lf (const char *in)
 	return out;
 }
 
+/**
+ * exchange_mapi_util_profile_name:
+ * @username: User name of the profile
+ * @domain: Domain name of the profile
+ *
+ * Constructs profile name from given @username and @domain and
+ * returns it as a newly allocated string.
+ **/
+gchar *
+exchange_mapi_util_profile_name (const gchar *username, const gchar *domain)
+{
+	gchar *res;
+
+	res = g_strdup_printf ("%s@%s", username, domain);
+
+	return g_strcanon (res, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@", '_');
+}
